# monte-carlo-integration.py - L1-Sciences Nice, Printemps 2017
# Calcul de l'integrale de f(x) = 1/(2+x^3) sur [0,1] par une methode de Monte-Carlo

###############################################
# Sans interface graphique, calcul uniquement #
###############################################
from random import random    # pour tirer des float dans [0,1[

def f(x) :
    return 1/(2+x**3)

def integrale(N) :     # avec n points aleatoires
    S = 0
    for i in range(N) :
        (x,y) = (random(), random())    # dans le rectangle ]0,1[ x ]0,1[
        if y < f(x) : S = S + 1
    return S / N

print('Test integrale Monte-Carlo 50000 points : ',end='')
print(integrale(50000))     # 0.45063. 
# On attend 0.450822 par Mathematica. Voir la fichier integrale.nb en Mathematica...

def integrale_riemann(f,a,b,N) :
    h = (b-a)/N                 # largeur de chaque sous-intervalle de [a,b]
    return h * sum(f(a+i*h) for i in range(N))

print('Test integrale Riemann : ',end='')
print(integrale_riemann(f,0,1,1000))     # 0.45090543481930634. On attend 0.450822 par Mathematica

##################################
# Avec interface graphique (GUI) #
##################################
# On n'utilise pas les fonctions integrale et integrale_riemann ci-dessus !

from tkinter import *

root = Tk()
root.title('Monte-Carlo Integration')
canvas = Canvas(root,width=400,height=400,bg='yellow')    # side = TOP par défaut
canvas.pack()
hpanel = Frame(root)   # un panneau horizontal invisible qui contiendra 2 widgets
label_N = Label(hpanel,text='nb_points =')
label_N.pack(side=LEFT)
entry_N = Entry(hpanel,width=5)
entry_N.pack(side=LEFT)
entry_N.insert(0,'500')
hpanel.pack()
res = StringVar()
label_res = Label(root,textvariable=res)    # on ne travaillera que sur res !
label_res.pack()
res.set('integrale = ?')

def segment(X1,Y1,X2,Y2) :  # les arguments sont en unites mathematiques
    # d'abord le changement de repere
    x1 = 400 * X1 ; y1 = 400 * (1 - Y1)
    x2 = 400 * X2 ; y2 = 400 * (1 - Y2)
    # puis le trace du segment dans le canvas
    canvas.create_line(x1,y1,x2,y2)
    
def plot() :    # 1 unite mathematique sur Ox = 400 pixels
    (X,Y) = (0,f(0))         # en unites mathematiques
    dX = 1/400
    for i in range(0,400) :  # pour chaque pixel sur Ox
        Xs = X + dX
        Ys = f(Xs)
        segment(X,Y,Xs,Ys)
        (X,Y) = (Xs,Ys)

def dessiner() :
    canvas.delete(ALL)
    plot()
    N = int(entry_N.get())
    S = 0
    for i in range(N) :
        (X,Y) = (random(),random())     # coordonnees mathematiques
        (x,y) = (X*400,400*(1-Y))       # coordonnees du canvas
        if Y < f(X) :                   # tombe-t-il dans l'aire cherchee ?
            S = S + 1
            canvas.create_oval(x,y,x,y,outline='red')
        else :
            canvas.create_oval(x,y,x,y,outline='black')
    proba = S / N
    res.set('integrale = {}'.format(proba))

button = Button(root,text='Go !',command=dessiner)
button.pack()
root.mainloop()




